@@ -1,16 +1,15 @@
 import asyncio
 import asyncio.events
 import errno
-import re
+import struct
 import sys
 import threading
 import weakref
 import zmq
 
-from collections import deque, Iterable
-from ipaddress import ip_address
+from collections import deque, Iterable, namedtuple
 
-from .interface import ZmqTransport
+from .interface import ZmqTransport, ZmqProtocol
 from .log import logger
 from .selector import ZmqSelector
 from .util import _EndpointsSet
@@ -25,6 +24,9 @@ else:
 __all__ = ['ZmqEventLoop', 'ZmqEventLoopPolicy', 'create_zmq_connection']
 
 
+SocketEvent = namedtuple('SocketEvent', 'event value endpoint')
+
+
 @asyncio.coroutine
 def create_zmq_connection(protocol_factory, zmq_type, *,
                           bind=None, connect=None, zmq_sock=None, loop=None):
@@ -193,16 +195,57 @@ class ZmqEventLoop(SelectorEventLoop):
             raise
 
 
+class _ZmqEventProtocol(ZmqProtocol):
+    """This protocol is used internally by aiozmq to receive messages
+    from a socket event monitor socket. This protocol decodes each event
+    message into a namedtuple and then passes them through to the
+    protocol running the socket that is being monitored via the
+    ZmqProtocol.event_received method.
+
+    This design simplifies the API visible to the developer at the cost
+    of adding some internal complexity - a hidden protocol that transfers
+    events from the monitor protocol to the monitored socket's protocol.
+    """
+
+    def __init__(self, loop, main_protocol):
+        self._protocol = main_protocol
+        self.wait_ready = asyncio.Future(loop=loop)
+        self.wait_closed = asyncio.Future(loop=loop)
+
+    def connection_made(self, transport):
+        self.transport = transport
+        self.wait_ready.set_result(True)
+
+    def connection_lost(self, exc):
+        self.wait_closed.set_result(exc)
+
+    def msg_received(self, data):
+        if len(data) != 2 or len(data[0]) != 6:
+            raise RuntimeError(
+                "Invalid event message format: {}".format(data))
+        event, value = struct.unpack("=hi", data[0])
+        endpoint = data[1].decode()
+        self.event_received(SocketEvent(event, value, endpoint))
+
+    def event_received(self, evt):
+        self._protocol.event_received(evt)
+
+
 class _BaseTransport(ZmqTransport):
 
-    _TCP_RE = re.compile('^tcp://(.+):(\d+)|\*$')
     LOG_THRESHOLD_FOR_CONNLOST_WRITES = 5
+    ZMQ_TYPES = {getattr(zmq, name): name
+                 for name in ('PUB', 'SUB', 'REP', 'REQ',
+                              'PUSH', 'PULL', 'DEALER', 'ROUTER',
+                              'XPUB', 'XSUB', 'PAIR', 'STREAM')
+                 if hasattr(zmq, name)}
 
     def __init__(self, loop, zmq_type, zmq_sock, protocol):
         super().__init__(None)
         self._protocol_paused = False
         self._set_write_buffer_limits()
         self._extra['zmq_socket'] = zmq_sock
+        self._extra['zmq_type'] = zmq_type
         self._loop = loop
         self._zmq_sock = zmq_sock
         self._zmq_type = zmq_type
@@ -215,6 +258,27 @@ class _BaseTransport(ZmqTransport):
         self._subscriptions = set()
         self._paused = False
         self._conn_lost = 0
+        self._monitor = None
+
+    def __repr__(self):
+        info = ['ZmqTransport',
+                'sock={}'.format(self._zmq_sock),
+                'type={}'.format(self.ZMQ_TYPES[self._zmq_type])]
+        try:
+            events = self._zmq_sock.getsockopt(zmq.EVENTS)
+            if events & zmq.POLLIN:
+                info.append('read=polling')
+            else:
+                info.append('read=idle')
+            if events & zmq.POLLOUT:
+                state = 'polling'
+            else:
+                state = 'idle'
+            bufsize = self.get_write_buffer_size()
+            info.append('write=<{}, bufsize={}>'.format(state, bufsize))
+        except zmq.ZMQError:
+            pass
+        return '<{}>'.format(' '.join(info))
 
     def write(self, data):
         if not data:
@@ -314,6 +378,9 @@ class _BaseTransport(ZmqTransport):
         self._high_water = high
         self._low_water = low
 
+    def get_write_buffer_limits(self):
+        return (self._low_water, self._high_water)
+
     def set_write_buffer_limits(self, high=None, low=None):
         self._set_write_buffer_limits(high=high, low=low)
         self._maybe_pause_protocol()
@@ -408,9 +475,6 @@ class _BaseTransport(ZmqTransport):
             if not isinstance(endpoint, str):
                 raise TypeError('endpoint should be str, got {!r}'
                                 .format(endpoint))
-            match = self._TCP_RE.match(endpoint)
-            if match:
-                ip_address(match.group(1))  # check for correct IPv4 or IPv6
             try:
                 self._zmq_sock.connect(endpoint)
             except zmq.ZMQError as exc:
@@ -463,6 +527,46 @@ class _BaseTransport(ZmqTransport):
             raise NotImplementedError("Not supported ZMQ socket type")
         return _EndpointsSet(self._subscriptions)
 
+    @asyncio.coroutine
+    def enable_monitor(self, events=None):
+
+        # The standard approach of binding and then connecting does not
+        # work in this specific case. The event loop does not properly
+        # detect messages on the inproc transport which means that event
+        # messages get missed.
+        # pyzmq's 'get_monitor_socket' method can't be used because this
+        # performs the actions in the wrong order for use with an event
+        # loop.
+        # For more information on this issue see:
+        # http://lists.zeromq.org/pipermail/zeromq-dev/2015-July/029181.html
+
+        if (zmq.zmq_version_info() < (4,) or
+                zmq.pyzmq_version_info() < (14, 4,)):
+            raise NotImplementedError(
+                "Socket monitor requires libzmq >= 4 and pyzmq >= 14.4, "
+                "have libzmq:{}, pyzmq:{}".format(
+                    zmq.zmq_version(), zmq.pyzmq_version()))
+
+        if self._monitor is None:
+            addr = "inproc://monitor.s-{}".format(self._zmq_sock.FD)
+            events = events or zmq.EVENT_ALL
+            _, self._monitor = yield from create_zmq_connection(
+                lambda: _ZmqEventProtocol(self._loop, self._protocol),
+                zmq.PAIR, connect=addr, loop=self._loop)
+            # bind must come after connect
+            self._zmq_sock.monitor(addr, events)
+            yield from self._monitor.wait_ready
+
+    @asyncio.coroutine
+    def disable_monitor(self):
+        self._disable_monitor()
+
+    def _disable_monitor(self):
+        if self._monitor:
+            self._zmq_sock.disable_monitor()
+            self._monitor.transport.close()
+            self._monitor = None
+
 
 class _ZmqTransportImpl(_BaseTransport):
 
@@ -529,6 +633,8 @@ class _ZmqTransportImpl(_BaseTransport):
         if self._closing:
             return
         self._closing = True
+        if self._monitor:
+            self._disable_monitor()
         if not self._paused:
             self._loop.remove_reader(self._zmq_sock)
         if not self._buffer:
@@ -538,6 +644,8 @@ class _ZmqTransportImpl(_BaseTransport):
     def _force_close(self, exc):
         if self._conn_lost:
             return
+        if self._monitor:
+            self._disable_monitor()
         if self._buffer:
             self._buffer.clear()
             self._buffer_size = 0
@@ -655,6 +763,8 @@ class _ZmqLooplessTransportImpl(_BaseTransport):
         if self._closing:
             return
         self._closing = True
+        if self._monitor:
+            self._disable_monitor()
         if not self._buffer:
             self._conn_lost += 1
             if not self._paused:
@@ -664,6 +774,8 @@ class _ZmqLooplessTransportImpl(_BaseTransport):
     def _force_close(self, exc):
         if self._conn_lost:
             return
+        if self._monitor:
+            self._disable_monitor()
         if self._buffer:
             self._buffer.clear()
             self._buffer_size = 0
