@@ -3,12 +3,11 @@ import asyncio
 import inspect
 import pprint
 import textwrap
-
-from aiozmq import interface
 from types import MethodType
 
-from .packer import _Packer
 from .log import logger
+from .packer import _Packer
+from aiozmq import interface
 
 
 class Error(Exception):
@@ -153,24 +152,21 @@ class _BaseProtocol(interface.ZmqProtocol):
         for waiter in self.done_waiters:
             waiter.set_result(None)
 
-    def add_pending(self, fut):
-        self.pending_waiters.add(fut)
-
-    def discard_pending(self, fut):
-        self.pending_waiters.discard(fut)
-
 
 class _BaseServerProtocol(_BaseProtocol):
 
     def __init__(self, loop, handler, *,
-                 translation_table=None, log_exceptions=False,
-                 exclude_log_exceptions=()):
+                 translation_table=None,
+                 log_exceptions=False,
+                 exclude_log_exceptions=(),
+                 timeout=None):
         super().__init__(loop, translation_table=translation_table)
         if not isinstance(handler, AbstractHandler):
-            raise TypeError('handler must implement AbstractHandler ABC')
+            raise TypeError('handler must implement AbstractHandler')
         self.handler = handler
         self.log_exceptions = log_exceptions
         self.exclude_log_exceptions = exclude_log_exceptions
+        self.timeout = timeout
 
     def connection_lost(self, exc):
         super().connection_lost(exc)
@@ -244,8 +240,17 @@ class _BaseServerProtocol(_BaseProtocol):
                     if isinstance(exc, e):
                         return
                 logger.exception(textwrap.dedent("""\
-                    An exception from method %r call occurred.
+                    An exception %r from method %r call occurred.
                     args = %s
                     kwargs = %s
                     """),
-                    name, pprint.pformat(args), pprint.pformat(kwargs))  # noqa
+                    exc, name,
+                    pprint.pformat(args), pprint.pformat(kwargs))  # noqa
+
+    def add_pending(self, coro):
+        fut = asyncio.async(coro, loop=self.loop)
+        self.pending_waiters.add(fut)
+        return fut
+
+    def discard_pending(self, fut):
+        self.pending_waiters.discard(fut)
