@@ -1,22 +1,16 @@
 """ZeroMQ RPC"""
-
 import asyncio
 import os
 import random
 import struct
-import time
 import sys
-
-import zmq
-
-
+import time
 from collections import ChainMap
 from functools import partial
 
+import zmq
 from aiozmq import create_zmq_connection
 
-from .log import logger
-
 from .base import (
     GenericError,
     NotFoundError,
@@ -26,6 +20,7 @@ from .base import (
     _BaseProtocol,
     _BaseServerProtocol,
     )
+from .log import logger
 from .util import (
     _MethodCall,
     _fill_error_table,
@@ -75,7 +70,7 @@ def connect_rpc(*, connect=None, bind=None, loop=None,
 @asyncio.coroutine
 def serve_rpc(handler, *, connect=None, bind=None, loop=None,
               translation_table=None, log_exceptions=False,
-              exclude_log_exceptions=()):
+              exclude_log_exceptions=(), timeout=None):
     """A coroutine that creates and connects/binds RPC server instance.
 
     Usually for this function you need to use *bind* parameter, but
@@ -91,6 +86,8 @@ def serve_rpc(handler, *, connect=None, bind=None, loop=None,
 
     translation_table -- an optional table for custom value translators.
 
+    timeout -- timeout for performing handling of async server calls.
+
     loop -- an optional parameter to point ZmqEventLoop instance.  If
             loop is None then default event loop will be given by
             asyncio.get_event_loop call.
@@ -105,7 +102,8 @@ def serve_rpc(handler, *, connect=None, bind=None, loop=None,
         lambda: _ServerProtocol(loop, handler,
                                 translation_table=translation_table,
                                 log_exceptions=log_exceptions,
-                                exclude_log_exceptions=exclude_log_exceptions),
+                                exclude_log_exceptions=exclude_log_exceptions,
+                                timeout=timeout),
         zmq.ROUTER, connect=connect, bind=bind, loop=loop)
     return Service(loop, proto)
 
@@ -219,12 +217,15 @@ class _ServerProtocol(_BaseServerProtocol):
     RESP_SUFFIX = struct.Struct('=Ld?')
 
     def __init__(self, loop, handler, *,
-                 translation_table=None, log_exceptions=False,
-                 exclude_log_exceptions=()):
+                 translation_table=None,
+                 log_exceptions=False,
+                 exclude_log_exceptions=(),
+                 timeout=None):
         super().__init__(loop, handler,
                          translation_table=translation_table,
                          log_exceptions=log_exceptions,
-                         exclude_log_exceptions=exclude_log_exceptions)
+                         exclude_log_exceptions=exclude_log_exceptions,
+                         timeout=timeout)
         self.prefix = self.RESP_PREFIX.pack(os.getpid() % 0x10000,
                                             random.randrange(0x10000))
 
@@ -250,8 +251,7 @@ class _ServerProtocol(_BaseServerProtocol):
             fut.set_exception(exc)
         else:
             if asyncio.iscoroutinefunction(func):
-                fut = asyncio.async(func(*args, **kwargs), loop=self.loop)
-                self.add_pending(fut)
+                fut = self.add_pending(func(*args, **kwargs))
             else:
                 fut = asyncio.Future(loop=self.loop)
                 try:
@@ -285,6 +285,6 @@ class _ServerProtocol(_BaseServerProtocol):
             prefix = self.prefix + self.RESP_SUFFIX.pack(req_id,
                                                          time.time(), True)
             exc_type = exc.__class__
-            exc_info = (exc_type.__module__ + '.' + exc_type.__name__,
+            exc_info = (exc_type.__module__ + '.' + exc_type.__qualname__,
                         exc.args, repr(exc))
             self.transport.write(pre + [prefix, self.packer.packb(exc_info)])
